------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2008 Vadim Godunko <vgodunko@gmail.com>                      --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 2733 $ $Date: 2008-05-14 08:54:18 +0400 (Wed, 14 May 2008) $
------------------------------------------------------------------------------
with Qt4.Generic_Arguments;
with Qt4.Generic_Return_Arguments;
limited with Qt4.Objects;

package Qt4.Meta_Objects is

   type Call is (Invoke_Meta_Method,
                 Read_Property,
                 Write_Property,
                 Reset_Property,
                 Query_Property_Designable,
                 Query_Property_Scriptable,
                 Query_Property_Stored,
                 Query_Property_Editable,
                 Query_Property_User);
   pragma Convention (C, Call);

   type Q_Meta_Object is record
      Super_Data  : access constant Q_Meta_Object;
      String_Data : access constant Interfaces.C.char;
      Data        : access constant Q_Unsigned_Integer;
      Extra_Data  : access Q_Meta_Object;
   end record;
   pragma Convention (C, Q_Meta_Object);

   function Class_Name (Self : not null access constant Q_Meta_Object)
     return String;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Connect_Slots_By_Name
   --!    <Purpose> Соединяет сигналы, со слотами, оъявленными в виде:
   --! procedure on_<мия виджета>_<имя сигнала> (параметры).
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   procedure Connect_Slots_By_Name
    (Object : not null access Qt4.Objects.Q_Object'Class);

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Invoke_Method
   --!    <Purpose> Связка для функции QMetaObject::invokeMethod.
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   procedure Invoke_Method
    (Object   : not null access Qt4.Objects.Q_Object'Class;
     Member   : in              String;
     The_Type : in              Qt4.Connection_Type;
     Returns  : in
       Qt4.Generic_Return_Arguments.Q_Generic_Return_Argument;
     Val_0    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_1    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_2    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_3    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_4    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_5    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_6    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_7    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_8    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_9    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument);

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Invoke_Method
   --!    <Purpose> Связка для функции QMetaObject::invokeMethod.
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   procedure Invoke_Method
    (Object   : not null access Qt4.Objects.Q_Object'Class;
     Member   : in              String;
     Returns  : in
       Qt4.Generic_Return_Arguments.Q_Generic_Return_Argument;
     Val_0    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_1    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_2    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_3    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_4    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_5    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_6    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_7    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_8    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_9    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument);

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Invoke_Method
   --!    <Purpose> Связка для функции QMetaObject::invokeMethod.
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   procedure Invoke_Method
    (Object   : not null access Qt4.Objects.Q_Object'Class;
     Member   : in              String;
     The_Type : in              Qt4.Connection_Type;
     Val_0    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_1    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_2    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_3    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_4    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_5    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_6    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_7    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_8    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_9    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument);

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Invoke_Method
   --!    <Purpose> Связка для функции QMetaObject::invokeMethod.
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   procedure Invoke_Method
    (Object   : not null access Qt4.Objects.Q_Object'Class;
     Member   : in              String;
     Val_0    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_1    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_2    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_3    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_4    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_5    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_6    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_7    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_8    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument;
     Val_9    : in              Qt4.Generic_Arguments.Q_Generic_Argument
       := Qt4.Generic_Arguments.No_Argument);

   package Internals is

      --  Пакет содержит недокументированные подпрограммы класса QMetaClass,
      --  используемые в частности компилятором метаинформации.

      ------------------------------------------------------------------------
      --! <Subprogram>
      --!    <Unit> Activate
      --!    <Purpose> Связка для функции QMetaObject::activate.
      --!    <Exceptions>
      ------------------------------------------------------------------------
      procedure Activate
       (Sender             : not null access Qt4.Objects.Q_Object'Class;
        Meta               : not null access constant Q_Meta_Object;
        Local_Signal_Index : in                       Q_Integer;
        Arguments          : in                       System.Address);

   end Internals;

end Qt4.Meta_Objects;
