------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2008 Vadim Godunko <vgodunko@gmail.com>                      --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 2733 $ $Date: 2008-05-14 08:54:18 +0400 (Wed, 14 May 2008) $
------------------------------------------------------------------------------
with Ada.Unchecked_Conversion;

package body Qt4_Generic_Flags is

   function To_Flags is new Ada.Unchecked_Conversion (Enum, Flags);

   ---------
   -- "+" --
   ---------

   function "+" (Right : in Enum) return Flags is
   begin
      return To_Flags (Right);
   end "+";

   ---------
   -- "+" --
   ---------

   function "+" (Left : in Enum; Right : in Enum) return Flags is
   begin
      return To_Flags (Left) or To_Flags (Right);
   end "+";

   ---------
   -- "+" --
   ---------

   function "+" (Left : in Flags; Right : in Enum) return Flags is
   begin
      return Left or To_Flags (Right);
   end "+";

   ---------
   -- "+" --
   ---------

   function "+" (Left : in Enum; Right : in Flags) return Flags is
   begin
      return To_Flags (Left) or Right;
   end "+";

   ---------
   -- "+" --
   ---------

   function "+" (Left : in Flags; Right : in Flags) return Flags is
   begin
      return Left or Right;
   end "+";

   ---------
   -- "-" --
   ---------

   function "-" (Left : in Flags; Right : in Enum) return Flags is
   begin
--      return Left and (Flags'Last xor To_Flags (Right));
      return Left and not To_Flags (Right);
   end "-";

   ---------
   -- "-" --
   ---------

   function "-" (Left : in Flags; Right : in Flags) return Flags is
   begin
      return Left and not Right;
   end "-";

   ---------
   -- "=" --
   ---------

   function "=" (Left : in Flags; Right : in Enum) return Boolean is
   begin
      return Left = To_Flags (Right);
   end "=";

   ---------
   -- "=" --
   ---------

   function "=" (Left : in Enum; Right : in Flags) return Boolean is
   begin
      return To_Flags (Left) = Right;
   end "=";

   ------------
   -- Is_Set --
   ------------

   function Is_Set (Item : in Flags; Value : in Enum) return Boolean is
   begin
      return (Item and To_Flags (Value)) = Value;
   end Is_Set;

end Qt4_Generic_Flags;
