------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2008, 2009 Vadim Godunko <vgodunko@gmail.com>                --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3363 $ $Date: 2009-05-09 15:42:59 +0400 (Sat, 09 May 2009) $
------------------------------------------------------------------------------
with Qt_Ada.Internals.Events.Registry;

package body Qt4.Focus_Events.Impl is

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> gotFocus_wrapper
   --!    <Purpose>
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   function gotFocus_wrapper
    (Self : not null access Qt4.Events.Impl.Thin.QEvent)
       return Qt4.bool;
   pragma Import
    (C, gotFocus_wrapper, "__Qt4Ada_QFocusEvent_gotFocus_wrapper");

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> lostFocus_wrapper
   --!    <Purpose>
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   function lostFocus_wrapper
    (Self : not null access Qt4.Events.Impl.Thin.QEvent)
       return Qt4.bool;
   pragma Import
    (C, lostFocus_wrapper, "__Qt4Ada_QFocusEvent_lostFocus_wrapper");

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> reason_wrapper
   --!    <Purpose>
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   function reason_wrapper (Self : not null access Qt4.Events.Impl.Thin.QEvent)
     return Qt4.Focus_Reason;
   pragma Import (C, reason_wrapper, "__Qt4Ada_QFocusEvent_reason_wrapper");

   function Create
    (Object : not null Qt_Ada.Internals.Events.QEvent_Access)
       return not null Qt4.Events.Q_Event_Access;

   ------------
   -- Create --
   ------------

   function Create
    (Object : not null Qt_Ada.Internals.Events.QEvent_Access)
       return not null Qt4.Events.Q_Event_Access
   is
   begin
      return Result : constant not null Qt4.Events.Q_Event_Access
        := new Q_Focus_Event_Impl
      do
         Qt_Ada.Internals.Events.Initialize (Result, Object);
      end return;
   end Create;

   ---------------
   -- Got_Focus --
   ---------------

   function Got_Focus (Self : not null access Q_Focus_Event_Impl)
     return Boolean
   is
   begin
      return Qt4.To_Boolean (gotFocus_wrapper (Self.QEvent_View));
   end Got_Focus;

   ----------------
   -- Lost_Focus --
   ----------------

   function Lost_Focus (Self : not null access Q_Focus_Event_Impl)
     return Boolean
   is
   begin
      return Qt4.To_Boolean (lostFocus_wrapper (Self.QEvent_View));
   end Lost_Focus;

   ------------
   -- Reason --
   ------------

   function Reason (Self : not null access Q_Focus_Event_Impl)
     return Qt4.Focus_Reason
   is
   begin
      return reason_wrapper (Self.QEvent_View);
   end Reason;

begin
   Qt_Ada.Internals.Events.Registry.Register
    (Qt4.Events.Focus_In, Create'Access);
   Qt_Ada.Internals.Events.Registry.Register
    (Qt4.Events.Focus_Out, Create'Access);
end Qt4.Focus_Events.Impl;
