------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2008, 2009 Vadim Godunko <vgodunko@gmail.com>                --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3363 $ $Date: 2009-05-09 15:42:59 +0400 (Sat, 09 May 2009) $
------------------------------------------------------------------------------
with Interfaces.C;

with Qt4.Events.Impl;
with Qt_Ada.Internals.Events.Registry;

package body Qt4.Key_Events.Impl is

   function Create
    (Object : not null Qt_Ada.Internals.Events.QEvent_Access)
       return not null Qt4.Events.Q_Event_Access;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> count_wrapper
   --!    <Purpose>
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   function count_wrapper
    (Cpp_Object : not null access Qt4.Events.Impl.Thin.QEvent)
      return Interfaces.C.int;
   pragma Import (C, count_wrapper, "__Qt4Ada_QKeyEvent_count_wrapper");

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> isAutoRepear_wrapper
   --!    <Purpose>
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   function isAutoRepear_wrapper
    (Cpp_Object : not null access Qt4.Events.Impl.Thin.QEvent)
      return Qt4.bool;
   pragma Import
    (C, isAutoRepear_wrapper, "__Qt4Ada_QKeyEvent_isAutoRepeat_wrapper");

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> key_wrapper
   --!    <Purpose>
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   function key_wrapper
    (Cpp_Object : not null access Qt4.Events.Impl.Thin.QEvent)
      return Qt4.Key;
   pragma Import
    (C, key_wrapper, "__Qt4Ada_QKeyEvent_key_wrapper");

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> key_wrapper
   --!    <Purpose>
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   function modifiers_wrapper
    (Cpp_Object : not null access Qt4.Events.Impl.Thin.QEvent)
      return Qt4.Keyboard_Modifier;
   pragma Import
    (C, modifiers_wrapper, "__Qt4Ada_QKeyEvent_modifiers_wrapper");

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> nativeModifiers_wrapper
   --!    <Purpose>
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   function nativeModifiers_wrapper
    (Cpp_Object : not null access Qt4.Events.Impl.Thin.QEvent)
      return Interfaces.C.unsigned;
   pragma Import
    (C, nativeModifiers_wrapper, "__Qt4Ada_QKeyEvent_nativeModifiers_wrapper");

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> nativeScanCode_wrapper
   --!    <Purpose>
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   function nativeScanCode_wrapper
    (Cpp_Object : not null access Qt4.Events.Impl.Thin.QEvent)
      return Interfaces.C.unsigned;
   pragma Import
    (C, nativeScanCode_wrapper, "__Qt4Ada_QKeyEvent_nativeScanCode_wrapper");

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> nativeVirtualKey_wrapper
   --!    <Purpose>
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   function nativeVirtualKey_wrapper
    (Cpp_Object : not null access Qt4.Events.Impl.Thin.QEvent)
      return Interfaces.C.unsigned;
   pragma Import (C,
                  nativeVirtualKey_wrapper,
                  "__Qt4Ada_QKeyEvent_nativeVirtualKey_wrapper");

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> text_wrapper
   --!    <Purpose>
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   function text_wrapper
    (Cpp_Object : not null access Qt4.Events.Impl.Thin.QEvent)
      return not null access Qt4.Strings.Internals.QString;
   pragma Import (C, text_wrapper, "__Qt4Ada_QKeyEvent_text_wrapper");

   overriding
   function Count (Self : not null access Q_Key_Event_Impl) return Q_Integer is
   begin
      return Q_Integer (count_wrapper (Self.QEvent_View));
   end Count;

   ------------
   -- Create --
   ------------

   function Create
    (Object : not null Qt_Ada.Internals.Events.QEvent_Access)
       return not null Qt4.Events.Q_Event_Access
   is
   begin
      return Result : constant not null Qt4.Events.Q_Event_Access
        := new Q_Key_Event_Impl
      do
         Qt_Ada.Internals.Events.Initialize (Result, Object);
      end return;
   end Create;

   --------------------
   -- Is_Auto_Repeat --
   --------------------

   overriding
   function Is_Auto_Repeat (Self : not null access Q_Key_Event_Impl)
     return Boolean
   is
   begin
      return Qt4.To_Boolean (isAutoRepear_wrapper (Self.QEvent_View));
   end Is_Auto_Repeat;

   ---------
   -- Key --
   ---------

   overriding
   function Key (Self : not null access Q_Key_Event_Impl) return Qt4.Key
   is
   begin
      return key_wrapper (Self.QEvent_View);
   end Key;

   ---------------
   -- Modifiers --
   ---------------

   overriding
   function Modifiers (Self : not null access Q_Key_Event_Impl)
     return Keyboard_Modifier
   is
   begin
      return modifiers_wrapper (Self.QEvent_View);
   end Modifiers;

   ----------------------
   -- Native_Modifiers --
   ----------------------

   overriding
   function Native_Modifiers (Self : not null access Q_Key_Event_Impl)
     return Q_Unsigned_Integer
   is
   begin
      return Q_Unsigned_Integer (nativeModifiers_wrapper (Self.QEvent_View));
   end Native_Modifiers;

   ----------------------
   -- Native_Scan_Code --
   ----------------------

   overriding
   function Native_Scan_Code (Self : not null access Q_Key_Event_Impl)
     return Q_Unsigned_Integer
   is
   begin
      return Q_Unsigned_Integer (nativeScanCode_wrapper (Self.QEvent_View));
   end Native_Scan_Code;

   ------------------------
   -- Native_Virtual_Key --
   ------------------------

   overriding
   function Native_Virtual_Key (Self : not null access Q_Key_Event_Impl)
     return Q_Unsigned_Integer
   is
   begin
      return Q_Unsigned_Integer (nativeVirtualKey_wrapper (Self.QEvent_View));
   end Native_Virtual_Key;

   ----------
   -- Text --
   ----------

   overriding
   function Text (Self : not null access Q_Key_Event_Impl)
     return Qt4.Strings.Q_String
   is
   begin
      return
        Qt4.Strings.Internals.Create_Wrapper
         (Qt4.Strings.Internals.QString_Access
           (text_wrapper (Self.QEvent_View)));
   end Text;

begin
   Qt_Ada.Internals.Events.Registry.Register
    (Qt4.Events.Key_Press, Create'Access);
   Qt_Ada.Internals.Events.Registry.Register
    (Qt4.Events.Key_Release, Create'Access);
   Qt_Ada.Internals.Events.Registry.Register
    (Qt4.Events.Shortcut_Override, Create'Access);
end Qt4.Key_Events.Impl;
